# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

from . import util


@util.os_parametrize("osxml", filter_media=True)
def test_existing_variants_for_media(osxml):
    os_variants = osxml.variants
    for media in osxml.medias:
        media_variants = media.variants
        for variant in media_variants:
            if variant not in os_variants:
                raise AssertionError(
                    "variants: media in OS %s has variant=%s "
                    "but declared variants are %s"
                    % (osxml.shortid, variant, os_variants)
                )


@util.os_parametrize("osxml", filter_trees=True)
def test_existing_variants_for_trees(osxml):
    os_variants = osxml.variants
    for tree in osxml.trees:
        tree_variants = tree.variants
        for variant in tree_variants:
            if variant not in os_variants:
                raise AssertionError(
                    "variants: tree in OS %s has variant=%s "
                    "but declared variants are %s"
                    % (osxml.shortid, variant, os_variants)
                )


@util.os_parametrize("osxml", filter_images=True)
def test_existing_variants_for_images(osxml):
    os_variants = osxml.variants
    for image in osxml.images:
        image_variants = image.variants
        for variant in image_variants:
            if variant not in os_variants:
                raise AssertionError(
                    "variants: media in OS %s has variant=%s "
                    "but declared variants are %s"
                    % (osxml.shortid, variant, os_variants)
                )



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

from . import util


@util.datamap_parametrize("datamapxml")
def test_duplicate_mappings(datamapxml):
    """
    Check that there are no duplicate mappings.
    """
    seen = set()
    duplicates = set(
        inval
        for (inval, _) in datamapxml.mapping_list
        if inval in seen or seen.add(inval)
    )
    assert duplicates == set()



#!/usr/bin/env python3

from pathlib import Path
import re
import subprocess

DOWNLOAD_URL = "https://releases.pagure.org/libosinfo"
DOWNLOAD_FORMAT = "tar.xz"

GITLAB_REPO_URL = "https://gitlab.com/libosinfo/osinfo-db"
GITLAB_BRANCH = "main"
GITLAB_CI_JOB = "publish"
GITLAB_CI_ARTIFACT_BASEURL = f"{GITLAB_REPO_URL}/-/jobs/artifacts/{GITLAB_BRANCH}/raw"
GITLAB_CI_ARTIFACT_NIGHTLY = (
    f"{GITLAB_CI_ARTIFACT_BASEURL}/osinfo-db-latest.tar.xz?job={GITLAB_CI_JOB}"
)

WEB_ROOT = Path("public")

GPG_KEYS = (
    "berrange.asc",
    "fidencio.asc",
    "toso.asc",
)

scripts_dir = Path(__file__).parent

WEB_ROOT.mkdir(exist_ok=True)

for key in GPG_KEYS:
    subprocess.check_call(["gpg", "--import", scripts_dir / "keys" / key])

tag_proc = subprocess.run(
    ["git", "tag"], check=True, capture_output=True, encoding="utf-8"
)

tags = tag_proc.stdout.strip().splitlines()

versions = []
for tag in tags:
    m = re.match(r"^v(\d{8})$", tag)
    if m is None:
        continue

    # Ensure that we abort if we see what looks like a release tag,
    # but without a signature, or with an unexpected user's signature
    subprocess.check_call(["git", "tag", "--verify", tag])

    version = m.group(1)
    versions.append(version)

    taginfo = WEB_ROOT / (version + ".json")

    with open(taginfo, "w") as fh:
        print(
            f"""{{
    "version": 1,
    "release": {{
        "version": "{version}",
        "archive": "{DOWNLOAD_URL}/osinfo-db-{version}.{DOWNLOAD_FORMAT}",
        "signature": "{DOWNLOAD_URL}/osinfo-db-{version}.{DOWNLOAD_FORMAT}.asc"
    }}
}}""",
            file=fh,
        )


nightly = WEB_ROOT / "nightly.json"

with open(nightly, "w") as fh:
    print(
        f"""{{
    "version": 1,
    "release": {{
        "archive": "{GITLAB_CI_ARTIFACT_NIGHTLY}"
    }}
}}""",
        file=fh,
    )

if versions:
    versions.sort()
    latest_version = versions[-1]
else:
    latest_version = "nightly"

latest_src = latest_version + ".json"
latest_dst = WEB_ROOT / "latest.json"

latest_dst.unlink(missing_ok=True)
latest_dst.symlink_to(latest_src)



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import contextlib
import copy
import logging
import os
from pathlib import Path
import re

import lxml.etree as ET
import pytest

from . import util
from . import osinfo


# parameters that have a default in libosinfo, so they need to be set
# even if they are not marked as required
params_with_default = {
    "l10n-keyboard": "en_US",
    "l10n-language": "en_US",
    "l10n-timezone": "America/New_York",
}


# parameters that may be marked as required: in this case, use the following
# replacements for them
params_known_replacements = {
    "admin-password": "th3_l33t+pwd!",  # notsecret
    "hardware-arch": "x86_64",
    "hostname": "test-os",
    "reg-product-key": "12345-ABCDE-67890-FGHIJ-12345",  # notsecret
    "user-login": "guest",
    "user-password": "guest@_l33t+pwd!",  # notsecret
    "user-realname": "Guest User",
}


class AutoYaSTSchemas:
    """
    Loader for locally available AutoYaST schemas.

    This class tries to look for schemas for AutoYaST, either as available
    in the yast2-schema-collection package in openSUSE, or in the location
    pointed by the $INTERNAL_OSINFO_DB_AUTOYAST_SCHEMAS environment variable
    (if set).

    The result of each lookup is cached (found or not).
    """

    def __init__(self):
        self.schemas = {}
        self.path = None

        candidate_paths = [
            # as available in yast2-schema-collection
            "/usr/share/YaST2/schema/autoyast/products/",
        ]
        # try checking using an environment variable: if this is set,
        # it is always checked first (making it possible to override
        # a system installation)
        env_path = os.environ.get("INTERNAL_OSINFO_DB_AUTOYAST_SCHEMAS", None)
        if env_path:
            candidate_paths.insert(0, env_path)
        base_path = next((p for p in candidate_paths if os.path.isdir(p)), None)
        if base_path:
            self.path = Path(base_path)

    def _get_schema_path(self, os_distro, os_version, arch):
        if os_distro == "opensuse":
            # unknown version: no way to tell which schema it follows
            if os_version == "unknown":
                return None
            # Factory is the development version / source of Tumbleweed,
            # so use the schemas of Tumbleweed for both
            if os_version in ["factory", "tumbleweed"]:
                distro_string = "tw"
            else:
                distro_string = "leap" + os_version
        elif os_distro in ["sle", "sles", "sled"]:
            m = re.fullmatch(r"(\d+)(?:\.(\d+))?", os_version)
            if m:
                major = int(m.group(1))
                if major < 12:
                    distro_string = "sles-%d" % major
                else:
                    distro_string = "sle-%d" % major
                if m.group(2):
                    distro_string += "-sp" + m.group(2)
        if not distro_string:
            logging.warning("autoyast: unknown distro: %s/%s", os_distro, os_version)
            return None

        distro_path = self.path / distro_string
        if not distro_path.is_dir():
            logging.warning("autoyast: OS directory not found: %s", distro_path)
            return None

        arch_path = distro_path / arch
        if not arch_path.is_dir():
            logging.warning("autoyast: arch directory not found: %s", arch_path)
            return None

        # profile.rng is the "entry point" of the schema
        schema_path = arch_path / "profile.rng"
        if not schema_path.is_file():
            logging.warning("autoyast: profile.rng not found in %s", arch_path)
            return None

        return schema_path

    def load_schema(self, osxml, arch):
        if self.path is None:
            return None

        # each schema may be different depending on the distribution,
        # the distribution version, and the architecture; hence, cache the
        # schemas based on all of these factors
        cache_key = (osxml.distro, osxml.version, arch)

        try:
            rng = self.schemas[cache_key]
        except KeyError:
            schema_path = self._get_schema_path(*cache_key)
            if schema_path:
                rng = ET.RelaxNG(ET.parse(schema_path.open("r")))
            else:
                rng = None
            self.schemas[cache_key] = rng
        return rng


@util.os_parametrize("osxml", filter_installscripts=True)
def test_install_scripts(osxml):
    for script in osxml.installscripts:
        assert util.DataFiles.installscripts()[script]


@util.os_parametrize("osxml", filter_media=True)
def test_media_install_scripts(osxml):
    for media in osxml.medias:
        for script in media.installscripts:
            assert util.DataFiles.installscripts()[script]


def _generate_initial_script_config_xml(
    osxml, script, install_source, media=None, tree=None
):
    # create the XML document with the config
    doc = ET.Element("install-script-config")

    def import_element(doc, xmlobj, tag):
        new_element = xmlobj.get_cloned_element()
        if new_element.tag != tag:
            new_element.tag = tag
        doc.append(new_element)
        return new_element

    # <os>
    import_element(doc, osxml, "os")
    # <script>
    script_doc = import_element(doc, script, "script")
    source_subtag = ET.SubElement(script_doc, "installation-source")
    if media:
        source_subtag.text = "media"
    elif tree:
        source_subtag.text = "network"
    # <media>
    if media:
        import_element(doc, media, "media")
    # <tree>
    if tree:
        import_element(doc, tree, "tree")

    # <config> element with the parameters
    config = {}
    for param in script.params:
        n = param.name
        v = None
        if n in params_with_default:
            v = params_with_default[n]
        elif param.policy == "required":
            v = params_known_replacements[n]
        else:
            continue
        value_map = param.value_map
        if value_map:
            with contextlib.suppress(KeyError):
                v = util.DataFiles.datamaps()[value_map][v]
            assert v
        config[n] = v
    if media:
        # extra elements for a media
        config["hardware-arch"] = media.arch
    if tree:
        # extra elements for a tree
        config["hardware-arch"] = tree.arch
    config_doc = ET.SubElement(doc, "config")
    for subtagname, subtagval in config.items():
        if subtagval is None:
            continue
        prop_subtag = ET.SubElement(config_doc, subtagname)
        prop_subtag.text = subtagval

    return doc


def _generate_testdata_for_methods(script, params_doc):
    ret = []

    for method in script.injection_methods:
        method_doc = copy.deepcopy(params_doc)
        injection_subtag = ET.SubElement(
            method_doc.find("script"), "preferred-injection-method"
        )
        injection_subtag.text = method
        ret.append((script, method_doc))
        # injection methods that support the "/command-line" template
        if method in ["disk", "initrd"]:
            cmdline_doc = copy.deepcopy(method_doc)
            cmdline_doc.tag = "command-line"
            ret.append((script, cmdline_doc))

    return ret


def _collect_for_script_and_media(osxml, script, media):
    params_doc = _generate_initial_script_config_xml(
        osxml, script, "media", media=media
    )

    return _generate_testdata_for_methods(script, params_doc)


def _collect_for_script_and_tree(osxml, script, tree):
    params_doc = _generate_initial_script_config_xml(
        osxml, script, "network", tree=tree
    )

    return _generate_testdata_for_methods(script, params_doc)


def _collect_scripts():
    ret = []

    for osxml in util.DataFiles.oses():
        combinations = []
        scripts = osxml.installscripts
        for media in osxml.medias:
            if not media.installer_script:
                continue
            if media.installscripts:
                media_scripts = media.installscripts
            else:
                media_scripts = scripts
            if not media_scripts:
                continue
            for script_id in media_scripts:
                script = util.DataFiles.installscripts()[script_id]
                items = [script, media]
                combinations.append(items)
        if scripts:
            for tree in osxml.trees:
                if not tree.url:
                    continue
                for script_id in scripts:
                    script = util.DataFiles.installscripts()[script_id]
                    items = [script, tree]
                    combinations.append(items)

        if combinations:
            ret.append(pytest.param(osxml, combinations, id=osxml.shortid))

    return ret


def _check_autoyast(osxml, autoyast_schemas, params_doc, xml):
    arch = params_doc.find("config/hardware-arch")
    assert arch is not None

    rng = autoyast_schemas.load_schema(osxml, arch.text)
    if rng is None:
        return

    rng.assertValid(xml)


@pytest.fixture(scope="module")
def autoyast_schemas():
    autoyast_schemas = AutoYaSTSchemas()
    return autoyast_schemas


@pytest.mark.parametrize("osxml,testdata", _collect_scripts())
def test_generate_scripts(osxml, testdata, autoyast_schemas):
    # We didn't call _collect_for_script_and... from _collect_scripts
    # as that creates massive memory usage for the whole duration of
    # the test suite
    fulltestdata = []
    for script, obj in testdata:
        if isinstance(obj, osinfo.Tree):
            items = _collect_for_script_and_tree(osxml, script, obj)
        else:
            items = _collect_for_script_and_media(osxml, script, obj)
        fulltestdata.extend(items)

    for script, params_doc in fulltestdata:
        template = script.template
        transform = ET.XSLT(template)
        transform_result = transform(params_doc)
        assert transform_result
        result = bytes(transform_result)
        assert result

        output_is_xml = (
            template.find("xsl:output", template.nsmap).get("method", "xml") == "xml"
        )

        if params_doc.tag == "install-script-config":
            # This is the actual install script
            if output_is_xml:
                # It is an XML document, check it can be parsed as such
                xml = ET.XML(result)
                if script.expected_filename == "autoinst.xml":
                    # Looks like it may be an AutoYaST file
                    _check_autoyast(osxml, autoyast_schemas, params_doc, xml)
        elif params_doc.tag == "command-line":
            # This is the kernel command line to use;
            # tolerate only one newline at the end (as it may happen when
            # the <xsl:output> is "xml")
            if output_is_xml and result.endswith(b"\n"):
                result = result[:-1]
            assert b"\n" not in result
            assert len(result) > 0



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import enum
import http
import logging

import pytest
import requests
import urllib3

from . import util


class UniqueSet(set):
    """
    Special set that is never equal to any other set.

    This particular subclass of set is useful only as value in enum,
    to ensure that enum items that have the same items are still considered
    different; this is because enum.Enum turns duplicate values as aliases
    for the first item.

    Yes, it's an evil hack.
    """

    def __eq__(self, _):
        return False


class UrlType(UniqueSet, enum.Enum):
    URL_ISO = {
        # proper ISO mimetype
        "application/x-cd-image",
        "application/x-iso9660-image",
        "application/x-iso",
        # generic data
        "application/octet-stream",
        "binary/octet-stream",
        # ISO files on archive.netbsd.org
        "text/plain",
        # a few openSUSE Live images
        "application/x-up-download",
    }
    URL_INITRD = {
        # generic data
        "application/octet-stream",
        # gzip-compressed
        "application/x-gzip",
    }
    URL_DISK_RAW = {}
    URL_DISK_QCOW2 = {
        # generic data
        "application/octet-stream",
        # qcow2 files on some mirrors
        "application-x-qemu-disk",
        # qcow2 files on fedoraproject.org mirrors; similar issue of
        # https://pagure.io/fedora-infrastructure/issue/10766
        "application/x-troff-man",
        # qcow2 files on some opensuse.org mirrors
        "text/plain",
    }
    URL_DISK_VMDK = {}
    URL_TREEINFO = {
        # generic data
        "application/octet-stream",
        # on some Fedora mirrors
        "text/plain",
    }
    URL_DISK_CONTAINERDISK = {
        # image manifest
        "application/vnd.docker.distribution.manifest.v1+json",
    }
    URL_KERNEL = {
        # generic data
        "application/octet-stream",
    }
    URL_TREE = {
        # HTML page of file listing
        "text/html",
        "application/xhtml+xml",
    }
    URL_DISK_VHDX = {}


image_formats_types = {
    "containerdisk": UrlType.URL_DISK_CONTAINERDISK,
    "qcow2": UrlType.URL_DISK_QCOW2,
    "raw": UrlType.URL_DISK_RAW,
    "vhdx": UrlType.URL_DISK_VHDX,
    "vmdk": UrlType.URL_DISK_VMDK,
}


def _transform_docker_url(url):
    """
    Transform docker:// url into a docker registry API call
    See: https://docs.docker.com/registry/spec/api/#existing-manifests
    """
    url_parts = url.split("/")
    url = f"http://{url_parts[2]}/v2/"
    for i in range(3, len(url_parts) - 1):
        url += f"{url_parts[i]}/"
    image, tag = url_parts[-1].split(":")
    url += f"{image}/manifests/{tag}"
    return url


def _check_url(session: requests.Session, url, url_type, real_url=None):
    logging.info("url: %s, type: %s", real_url if real_url else url, url_type)
    headers = {"user-agent": "Wget/1.0"}
    try:
        response = session.head(url, allow_redirects=True, headers=headers, timeout=30)
    except requests.RequestException as e:
        # do not use logging.exception() here, as there is no need to log
        # the full traceback from requests
        logging.error("head: %s", e)
        return False
    content_type = response.headers.get("content-type")
    if content_type:
        try:
            content_type = content_type[0 : content_type.index(";")]
        except ValueError:
            pass

    msg = "response: %s; code: %d; content-type: %s; url: %s" % (
        http.client.responses[response.status_code],
        response.status_code,
        content_type,
        response.url,
    )

    if not response.ok:
        logging.error(msg)
        return False
    if content_type and content_type not in url_type.value:
        logging.error(msg)
        return False

    logging.info(msg)
    return True


def _collect_os_urls():
    """
    Iterate the OS list and return a list of pairs (shortid, [url list)
    """
    ret = []

    for osxml in util.DataFiles.oses():
        urls = []
        urls.extend(
            [(i.url, image_formats_types[i.format]) for i in osxml.images if i.url]
        )
        urls.extend([(m.url, UrlType.URL_ISO) for m in osxml.medias if m.url])
        for t in osxml.trees:
            if not t.url:
                continue
            urls.append((t.url, UrlType.URL_TREE))
            url = t.url
            if not url.endswith("/"):
                url += "/"
            if t.kernel:
                urls.append((url + t.kernel, UrlType.URL_KERNEL))
            if t.initrd:
                urls.append((url + t.initrd, UrlType.URL_INITRD))
            if t.treeinfo:
                urls.append((url + ".treeinfo", UrlType.URL_TREEINFO))
        if urls:
            urls_http = []
            urls_docker = []
            urls_other = []
            for url_pair in urls:
                u = url_pair[0]
                if u.startswith("http://") or u.startswith("https://"):
                    urls_http.append(url_pair)
                elif u.startswith("docker://"):
                    urls_docker.append(url_pair)
                else:
                    urls_other.append(url_pair)
            assert len(urls_http) + len(urls_docker) + len(urls_other) == len(urls)
            ret.append(
                pytest.param(urls_http, urls_docker, urls_other, id=osxml.shortid)
            )

    return ret


@pytest.fixture(scope="module")
def session():
    session = requests.Session()
    # As some distro URLs are flaky or rate limit requests (429) use Retry to
    # help navigate those at the expense of taking longer.
    # Use an high value for pool_connections: this represents the number of
    # urllib HTTP connection pools instantiated, each for a different host:
    # this way, we can reuse more connections across OSes.
    retries = urllib3.util.Retry(backoff_factor=1, status_forcelist=(429,), total=6)
    adapter = requests.adapters.HTTPAdapter(max_retries=retries, pool_connections=100)
    session.mount("https://", adapter)
    session.mount("http://", adapter)
    return session


@pytest.mark.parametrize("urls_http,urls_docker,urls_other", _collect_os_urls())
def test_urls(urls_http, urls_docker, urls_other, session):
    broken = []
    for url, url_type in urls_http:
        ok = _check_url(session, url, url_type)

        if not ok:
            broken.append(url)
    for url, url_type in urls_docker:
        http_url = _transform_docker_url(url)
        ok = _check_url(session, http_url, url_type, real_url=url)

        if not ok:
            broken.append(url)
    for url, _ in urls_other:
        logging.warning("unhandled URL: %s", url)
        broken.append(url)
    assert broken == []



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import lxml.etree as ET
import pytest

from . import util


@pytest.fixture(scope="module")
def schema():
    schema = ET.RelaxNG(ET.parse(util.DataFiles.schema.open("r")))
    return schema


@pytest.mark.parametrize("path", util.DataFiles.xmls())
def test_schema(path, schema):
    if not schema.validate(ET.parse(path.open("r"))):
        # pylint: disable=no-member
        raise AssertionError(str(schema.error_log.last_error))


@pytest.mark.parametrize("path", util.SourceFiles.xmls(), ids=lambda p: p.name)
def test_translations_in_source_files(path):
    xml = ET.parse(path.open("r"))
    assert xml.xpath("//*[@xml:lang]") == []



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import configparser
import contextlib
import logging
from pathlib import Path
import re
import pytest

from . import util
from . import isodata


def _load_languages(isofile):
    assert isofile.suffix == ".txt"
    langfile = isofile.with_suffix(".lng")
    config = configparser.ConfigParser()
    try:
        with langfile.open("r") as f:
            config.read_file(f)
    except FileNotFoundError:
        return None
    return set(config.get("general", "l10n-language").split(","))


def _get_isodatapaths():
    """
    Collect iso media data and return a list of tuples:
        (osname, isodatapaths)
    """
    this = Path(__file__).resolve()
    isodata_path = Path(this.parent, "isodata")

    ret = []
    osdirs = [d for d in isodata_path.glob(str(Path("*", "*"))) if d.is_dir()]

    for osdir in sorted(osdirs, key=util.path_sort):
        isofilepaths = list(osdir.glob("*.txt"))
        if len(isofilepaths):
            isodatapaths = []
            for isofile in isofilepaths:
                isodatapaths.append((isofile, _load_languages(isofile)))
            ret.append(pytest.param(osdir.name, isodatapaths, id=osdir.name))

    return ret


def _get_iso_languages(iso, isodatamedia):
    languages = set()
    regex = None
    l10n_language_map = None

    for l10n_language in iso.l10n_languages:
        value = l10n_language.value
        if l10n_language.regex:
            regex = value
            if l10n_language.l10n_language_map:
                l10n_language_map = l10n_language.l10n_language_map
        else:
            languages.add(value)

    # This matches the same logic in libosinfo (set_languages_for_media()
    # in OsinfoDb): if there is a regex specification for the language,
    # the result is used overring any other language previously specified
    if regex and isodatamedia.volumeid:
        m = re.match(regex, isodatamedia.volumeid)
        if m and m.lastindex:
            lang = m.group(1)
            with contextlib.suppress(KeyError):
                lang = util.DataFiles.datamaps()[l10n_language_map][lang]
            return {lang}

    return languages


@pytest.mark.parametrize("osname,isodatapaths", _get_isodatapaths())
def test_iso_detection(osname, isodatapaths):
    for isodatapath, languages in isodatapaths:
        # A symlink indicates an ISO from another version
        # that is ambiguous and so accidentally matches
        # this distro too. We ignore them here, because
        # we'll validate against the primary OS version
        if isodatapath.is_symlink():
            continue

        detected = []
        isodatamedia = isodata.get_isodatamedia(isodatapath)
        for osxml2 in util.DataFiles.oses():
            for media in osxml2.medias:
                if isodatamedia.match(media.iso):
                    if osname != osxml2.shortid:
                        logging.warning(
                            "ISO '%s' was matched by OS '%s' while it "
                            "should only be matched by OS '%s'",
                            isodatamedia.path,
                            osxml2.shortid,
                            osname,
                        )
                    else:
                        logging.info(
                            "ISO '%s' matched by OS '%s'",
                            isodatamedia.path,
                            osxml2.shortid,
                        )
                        if languages:
                            # If there are languages specified for this ISO,
                            # check they match what described in <iso>
                            iso_languages = _get_iso_languages(media.iso, isodatamedia)
                            assert languages == iso_languages
                    # For several distros we do not have the volume-size
                    # set as part of our DB, thus multiple detections may
                    # occur. Although this case is not the optimal, as long
                    # as we detect the very same distro it's okay-ish.
                    if osxml2.shortid not in detected:
                        detected.append(osxml2.shortid)

        if not detected:
            raise AssertionError(
                "isodata unmatched: %s [expected=%s]" % (isodatapath, osname)
            )

        ok = True
        for osid in detected:
            # Primary (correct) match
            if osid == osname:
                continue

            # Ambiguous (incorrect) matches, where we can't
            # distinguish from a different distro version
            link = Path(isodatapath.parent.parent, osid, isodatapath.name)
            if not link.exists():
                ok = False
        if ok:
            continue

        raise AssertionError(
            "isodata: %s\nMatched=%s but expected=%s"
            % (isodatapath, detected, [osname])
        )



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import collections
import logging
import re

from . import util


@util.os_parametrize("osxml")
def test_validate_short_ids(osxml):
    invalids = []

    pattern = r"[a-z0-9\.\-]"
    for shortid in osxml.shortids:
        result = re.match(pattern, shortid)
        if not result:
            invalids.append(shortid)

    assert invalids == []


def test_duplicate_short_ids():
    ids = collections.defaultdict(set)
    # collect the mapping of short-id -> IDs that have it
    for osxml in util.DataFiles.oses():
        for shortid in osxml.shortids:
            ids[shortid].add(osxml.internal_id)
    # drop all the mappings with only one ID
    for shortid in list(ids.keys()):
        if len(ids[shortid]) == 1:
            del ids[shortid]
    # log all the problematic ones
    for shortid, ids in ids.items():
        logging.info("shortid=%s is used by %s", shortid, sorted(list(ids)))
    assert ids == {}
